
//--------------------------------------------------------------------------------------------------
#include "shared.fxh"

//--------------------------------------------------------------------------------------------------
texture Texture0;
texture Texture1;

//--------------------------------------------------------------------------------------------------
sampler2D g_DiffuseSampler = sampler_state
{ 
    Texture				= ( Texture0 );
    MipFilter			= LINEAR;
    MinFilter			= LINEAR;
    MagFilter			= LINEAR;
};
//--------------------------------------------------------------------------------------------------

samplerCUBE g_EnvironmenteSampler = sampler_state
{ 
    Texture				= ( Texture1 );
    MipFilter			= LINEAR;
    MinFilter			= LINEAR;
    MagFilter			= LINEAR;
};
//--------------------------------------------------------------------------------------------------
// VertexShader I/O
//--------------------------------------------------------------------------------------------------
struct VS_INPUT
{
	float3 vPositionOS							: POSITION;						///< object space position
	float3 vNormalOS							: NORMAL0;						///< object space normal
	float2 vTexCoord							: TEXCOORD0;					///< texture coordinates
};
struct VS_OUTPUT
{
	float4 vPositionPS							: POSITION;						///< projection space position
	float2 vTexCoord							: TEXCOORD0;					///< texture coordinates
	float3 vPositionEnv							: TEXCOORD1;					///< object space position
	float3 vNormalOS							: TEXCOORD2;					///< object space normal
};
//--------------------------------------------------------------------------------------------------
float4 psMain( VS_OUTPUT i ) : COLOR0
{
	float4	vAmbientMaterial		= tex2D( g_DiffuseSampler, i.vTexCoord );
	float4	vEnvMaterial			= texCUBE( g_EnvironmenteSampler, i.vPositionEnv );
	float4	color					= vEnvMaterial * 0.3f + 0.7f * vAmbientMaterial;

	return color;
}
//--------------------------------------------------------------------------------------------------
VS_OUTPUT vsMain( VS_INPUT i )
{
	// preshader->
	const float4x4	mViewProjection			= mul( g_mView, g_mProjection );
	const float4x4	mWorldViewProjection	= mul( g_mWorld, mViewProjection );
	// <-preshader

	VS_OUTPUT o;

	o.vNormalOS			= i.vNormalOS;
	o.vTexCoord			= i.vTexCoord;
	float4 pos			= float4( i.vPositionOS, 1 );
	o.vPositionPS		= mul( pos, mWorldViewProjection );
	o.vPositionEnv		= -reflect( normalize( g_vCameraPos - pos ), i.vNormalOS  );

	return o;
}
//--------------------------------------------------------------------------------------------------
technique environment
{
    pass P0
    {
		VertexShader	= compile vs_2_0 vsMain();
        PixelShader		= compile ps_2_0 psMain();
    }
}
//--------------------------------------------------------------------------------------------------